use serde::{Deserialize, Serialize};

use crate::s2s::Dispatch;

#[derive(Debug, Serialize, Deserialize)]
#[serde(tag = "t", content = "d", rename_all = "SCREAMING_SNAKE_CASE")]
pub enum Payload {
    #[serde(rename = "D")]
    Dispatch {
        #[serde(rename = "e")]
        event: Dispatch,
        #[serde(rename = "s")]
        server_id: String,
    },
    Error {
        message: String,
    },
    /// Hello message, sent after authentication succeeds
    Hello {
        heartbeat_interval: u64,
        guilds: Vec<String>,
    },
    Identify {
        token: String,
    },
    /// Sent on a regular interval by the client, to keep the connection alive.
    Heartbeat {
        #[serde(rename = "t")]
        timestamp: u64,
    },
    /// Sent in response to a Heartbeat.
    HeartbeatAck {
        #[serde(rename = "t")]
        timestamp: u64,
    }
}
